/* OGMRip - A library for dvd video ripping and encoding
 * Copyright (C) 2004 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-backend.h"
#include "ogmsp-enums.h"
#include "ogmsp-exec.h"
#include "ogmrip-mux.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#define OGMRIP_MUX_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_MUX, OGMRipMuxPriv))

struct _OGMRipMuxPriv
{
  gchar *title;
  gchar *output;

  gchar *video;
  GList *audio;
  GList *chapters;
  GList *subp;
};

static void ogmrip_mux_finalize (GObject    *gobject);

G_DEFINE_ABSTRACT_TYPE (OGMRipMux, ogmrip_mux, OGMSP_TYPE_BIN)

static void
ogmrip_mux_class_init (OGMRipMuxClass *klass)
{
  GObjectClass *gobject_class;
  OGMSpSpawnClass *spawn_class;

  gobject_class = G_OBJECT_CLASS (klass);
  spawn_class = OGMSP_SPAWN_CLASS (klass);

  gobject_class->finalize = ogmrip_mux_finalize;

  g_type_class_add_private (klass, sizeof (OGMRipMuxPriv));
}

static void
ogmrip_mux_init (OGMRipMux *mux)
{
  mux->priv = OGMRIP_MUX_GET_PRIVATE (mux);
  mux->priv->chapters = NULL;
}

static void
ogmrip_stream_free (gchar **strv)
{
  g_free (strv[0]);
  g_free (strv);
}

static void
ogmrip_mux_finalize (GObject *gobject)
{
  OGMRipMux *mux;

  mux = OGMRIP_MUX (gobject);

  g_free (mux->priv->title);
  g_free (mux->priv->output);
  g_free (mux->priv->video);

  g_list_foreach (mux->priv->audio, (GFunc) ogmrip_stream_free, NULL);
  g_list_free (mux->priv->audio);

  g_list_foreach (mux->priv->chapters, (GFunc) ogmrip_stream_free, NULL);
  g_list_free (mux->priv->chapters);

  g_list_foreach (mux->priv->subp, (GFunc) ogmrip_stream_free, NULL);
  g_list_free (mux->priv->subp);

  G_OBJECT_CLASS (ogmrip_mux_parent_class)->finalize (gobject);
}

G_CONST_RETURN gchar *
ogmrip_mux_get_output (OGMRipMux *mux)
{
  g_return_val_if_fail (OGMRIP_IS_MUX (mux), NULL);

  return mux->priv->output;
}

void
ogmrip_mux_set_output (OGMRipMux *mux, const gchar *output)
{
  g_return_if_fail (OGMRIP_IS_MUX (mux));
  g_return_if_fail (output && *output);

  g_free (mux->priv->output);
  mux->priv->output = g_strdup (output);
}

void
ogmrip_mux_set_title (OGMRipMux *mux, const gchar *title)
{
  g_return_if_fail (OGMRIP_IS_MUX (mux));

  g_free (mux->priv->title);
  if (title)
    mux->priv->title = g_strdup (title);
  else
    mux->priv->title = NULL;
}

G_CONST_RETURN gchar *
ogmrip_mux_get_title (OGMRipMux *mux)
{
  g_return_val_if_fail (OGMRIP_IS_MUX (mux), NULL);

  return mux->priv->title;
}

G_CONST_RETURN gchar *
ogmrip_mux_get_video (OGMRipMux *mux)
{
  g_return_val_if_fail (OGMRIP_IS_MUX (mux), NULL);

  return mux->priv->video;
}

void
ogmrip_mux_set_video (OGMRipMux *mux, const gchar *input)
{
  g_return_if_fail (OGMRIP_IS_MUX (mux));
  g_return_if_fail (input && *input);

  g_free (mux->priv->video);
  mux->priv->video = g_strdup (input);
}

void
ogmrip_mux_add_audio (OGMRipMux *mux, const gchar *input, gint language)
{
  gchar **strv;

  g_return_if_fail (OGMRIP_IS_MUX (mux));
  g_return_if_fail (input && *input);

  strv = g_new0 (gchar *, 3); 

  strv[0] = g_strdup (input);
  strv[1] = GINT_TO_POINTER (language);

  mux->priv->audio = g_list_append (mux->priv->audio, strv);
}

void
ogmrip_mux_add_chapters (OGMRipMux *mux, const gchar *input, gint language)
{
  gchar **strv;

  g_return_if_fail (OGMRIP_IS_MUX (mux));
  g_return_if_fail (input && *input);

  strv = g_new0 (gchar *, 3); 

  strv[0] = g_strdup (input);
  strv[1] = GINT_TO_POINTER (language);

  mux->priv->chapters = g_list_append (mux->priv->chapters, strv);
}

void
ogmrip_mux_add_subp (OGMRipMux *mux, const gchar *input, gint language)
{
  gchar **strv;

  g_return_if_fail (OGMRIP_IS_MUX (mux));
  g_return_if_fail (input && *input);

  strv = g_new0 (gchar *, 3); 

  strv[0] = g_strdup (input);
  strv[1] = GINT_TO_POINTER (language);

  mux->priv->subp = g_list_append (mux->priv->subp, strv);
}

void
ogmrip_mux_foreach_audio (OGMRipMux *mux, OGMRipMuxFunc func, gpointer data)
{
  GList *link;
  gchar **strv;
  struct stat buf; 

  g_return_if_fail (OGMRIP_IS_MUX (mux));
  g_return_if_fail (func != NULL);

  for (link = mux->priv->audio; link; link = link->next)
  {
    strv = (gchar **) link->data;
    if (g_file_test (strv[0], G_FILE_TEST_IS_REGULAR))
      if (stat (strv[0], &buf) == 0 && buf.st_size > 0)
        func (mux, strv[0], GPOINTER_TO_INT (strv[1]), data);
  }
}

void
ogmrip_mux_foreach_chapter (OGMRipMux *mux, OGMRipMuxFunc func, gpointer data)
{
  GList *link;
  gchar **strv;
  struct stat buf; 

  g_return_if_fail (OGMRIP_IS_MUX (mux));
  g_return_if_fail (func != NULL);

  for (link = mux->priv->chapters; link; link = link->next)
  {
    strv = (gchar **) link->data;
    if (g_file_test (strv[0], G_FILE_TEST_IS_REGULAR))
      if (stat (strv[0], &buf) == 0 && buf.st_size > 0)
        func (mux, strv[0], GPOINTER_TO_INT (strv[1]), data);
  }
}

void
ogmrip_mux_foreach_subp (OGMRipMux *mux, OGMRipMuxFunc func, gpointer data)
{
  GList *link;
  gchar **strv;
  struct stat buf; 

  g_return_if_fail (OGMRIP_IS_MUX (mux));
  g_return_if_fail (func != NULL);

  for (link = mux->priv->subp; link; link = link->next)
  {
    strv = (gchar **) link->data;
    if (g_file_test (strv[0], G_FILE_TEST_IS_REGULAR))
      if (stat (strv[0], &buf) == 0 && buf.st_size > 0)
        func (mux, strv[0], GPOINTER_TO_INT (strv[1]), data);
  }
}

