/* OGMSpawn - A library to spawn processes
 * Copyright (C) 2004 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmsp-bin.h"
#include "ogmsp-enums.h"

#define OGMSP_BIN_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMSP_TYPE_BIN, OGMSpBinPriv))

struct _OGMSpBinPriv
{
  OGMSpSpawn *child;
};

static gint ogmsp_bin_run      (OGMSpSpawn     *spawn);
static void ogmsp_bin_add      (OGMSpContainer *container,
                                OGMSpSpawn     *child);
static void ogmsp_bin_remove   (OGMSpContainer *container,
                                OGMSpSpawn     *child);
static void ogmsp_bin_forall   (OGMSpContainer *container,
                                OGMSpCallback  callback,
                                gpointer       data);

G_DEFINE_ABSTRACT_TYPE (OGMSpBin, ogmsp_bin, OGMSP_TYPE_CONTAINER)

static void
ogmsp_bin_class_init (OGMSpBinClass *klass)
{
  OGMSpSpawnClass *spawn_class;
  OGMSpContainerClass *container_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);
  container_class = OGMSP_CONTAINER_CLASS (klass);

  spawn_class->run = ogmsp_bin_run;

  container_class->add = ogmsp_bin_add;
  container_class->remove = ogmsp_bin_remove;
  container_class->forall = ogmsp_bin_forall;

  g_type_class_add_private (klass, sizeof (OGMSpBinPriv));
}

static void
ogmsp_bin_init (OGMSpBin *bin)
{
  bin->priv = OGMSP_BIN_GET_PRIVATE (bin);
  bin->priv->child = NULL;
}

static gint
ogmsp_bin_run (OGMSpSpawn *spawn)
{
  OGMSpBin *bin;

  bin = OGMSP_BIN (spawn);
  if (bin->priv->child)
    return ogmsp_spawn_run (bin->priv->child);

  return OGMSP_RESULT_COMPLETED;
}

static void
ogmsp_bin_child_progress (OGMSpContainer *container, gdouble fraction, OGMSpSpawn *child)
{
  g_signal_emit_by_name (container, "progress", fraction);
}

static void
ogmsp_bin_add (OGMSpContainer *container, OGMSpSpawn *child)
{
  OGMSpBin *bin;

  bin = OGMSP_BIN (container);
  if (bin->priv->child == NULL)
  {
    guint handler;

    g_object_ref (child);
    bin->priv->child = child;

    handler = g_signal_connect_swapped (child, "progress", 
        G_CALLBACK (ogmsp_bin_child_progress), container);
    g_object_set_data (G_OBJECT (child), "__child_progress_handler__", 
        GUINT_TO_POINTER (handler));
  }
}

static void
ogmsp_bin_remove (OGMSpContainer *container, OGMSpSpawn *child)
{
  OGMSpBin *bin;

  bin = OGMSP_BIN (container);
  if (bin->priv->child == child)
  {
    gpointer handler;

    handler = g_object_get_data (G_OBJECT (child), "__child_progress_handler__");

    if (handler)
      g_signal_handler_disconnect (child, GPOINTER_TO_UINT (handler));

    g_object_unref (child);
    bin->priv->child = NULL;
  }
}

static void
ogmsp_bin_forall (OGMSpContainer *container, OGMSpCallback callback, gpointer data)
{
  OGMSpBin *bin;

  bin = OGMSP_BIN (container);
  if (callback && bin->priv->child)
    (* callback) (bin->priv->child, data);
}

OGMSpSpawn *
ogmsp_bin_get_child (OGMSpBin *bin)
{
  g_return_val_if_fail (OGMSP_IS_BIN (bin), NULL);

  return bin->priv->child;
}

