/* OGMSpawn - A library to spawn processes
 * Copyright (C) 2004 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmsp-queue.h"
#include "ogmsp-enums.h"

#define OGMSP_QUEUE_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMSP_TYPE_QUEUE, OGMSpQueuePriv))

struct _OGMSpQueuePriv
{
  gdouble progress;
  guint nchildren;
  guint completed;
};

static gint ogmsp_queue_run    (OGMSpSpawn     *spawn);
static void ogmsp_queue_add    (OGMSpContainer *container,
                                OGMSpSpawn     *child);
static void ogmsp_queue_remove (OGMSpContainer *container,
                                OGMSpSpawn     *child);

G_DEFINE_TYPE (OGMSpQueue, ogmsp_queue, OGMSP_TYPE_LIST)

static void
ogmsp_queue_class_init (OGMSpQueueClass *klass)
{
  OGMSpSpawnClass *spawn_class;
  OGMSpContainerClass *container_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);
  container_class = OGMSP_CONTAINER_CLASS (klass);

  spawn_class->run = ogmsp_queue_run;

  container_class->add = ogmsp_queue_add;
  container_class->remove = ogmsp_queue_remove;

  g_type_class_add_private (klass, sizeof (OGMSpQueuePriv));
}

static void
ogmsp_queue_init (OGMSpQueue *queue)
{
  queue->priv = OGMSP_QUEUE_GET_PRIVATE (queue);
}

static gint
ogmsp_queue_run (OGMSpSpawn *spawn)
{
  gint result = OGMSP_RESULT_COMPLETED;
  OGMSpQueue *queue;
  GList *child, *children;

  queue = OGMSP_QUEUE (spawn);

  children = ogmsp_list_get_children (OGMSP_LIST (spawn));
  queue->priv->nchildren = g_list_length (children);
  queue->priv->completed = 0;

  for (child = children; child; child = child->next)
  {
    if (result != OGMSP_RESULT_COMPLETED)
      break;

    ogmsp_spawn_set_async (OGMSP_SPAWN (child->data), FALSE);
    result = ogmsp_spawn_run (OGMSP_SPAWN (child->data));
    queue->priv->completed ++;
  }
  g_list_free (children);

  return result;
}

static void
ogmsp_queue_child_progress (OGMSpQueue *queue, gdouble fraction, OGMSpSpawn *child)
{
  fraction = (queue->priv->completed + fraction) / (gdouble) queue->priv->nchildren;
  g_signal_emit_by_name (queue, "progress", fraction);
}

static void
ogmsp_queue_add (OGMSpContainer *container, OGMSpSpawn *child)
{
  guint handler;

  handler = g_signal_connect_swapped (child, "progress", 
      G_CALLBACK (ogmsp_queue_child_progress), container);
  g_object_set_data (G_OBJECT (child), "__child_progress_handler__", 
      GUINT_TO_POINTER (handler));

  OGMSP_CONTAINER_CLASS (ogmsp_queue_parent_class)->add (container, child);
}

static void
ogmsp_queue_remove (OGMSpContainer *container, OGMSpSpawn *child)
{
  gpointer handler;

  handler = g_object_get_data (G_OBJECT (child), "__child_progress_handler__");
  if (handler)
    g_signal_handler_disconnect (container, GPOINTER_TO_UINT (handler));

  OGMSP_CONTAINER_CLASS (ogmsp_queue_parent_class)->remove (container, child);
}

OGMSpSpawn *
ogmsp_queue_new (void)
{
  return g_object_new (OGMSP_TYPE_QUEUE, NULL);
}

