/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"

#include <string.h>
#include <glib/gi18n.h>

#include <gtk/gtkmenu.h>
#include <gtk/gtkcombobox.h>
#include <gtk/gtkliststore.h>
#include <gtk/gtkcelllayout.h>
#include <gtk/gtkcellrenderertext.h>

#include "ogmrip-dvd-chooser.h"

enum
{
  DEVICE_CHANGED,
  LAST_SIGNAL
};

enum
{
  PROP_0,
  PROP_DEVICE
};

struct OGMRipDvdChooserPrivate
{
  GList *cdroms;
};

static void ogmrip_dvd_chooser_init         (OGMRipDvdChooser *chooser);
static void ogmrip_dvd_chooser_set_property (GObject          *object,
                                             guint            property_id,
                                             const GValue     *value,
                                             GParamSpec       *pspec);
static void ogmrip_dvd_chooser_get_property (GObject          *object,
                                             guint            property_id,
                                             GValue           *value,
                                             GParamSpec       *pspec);
static void ogmrip_dvd_chooser_finalize     (GObject          *object);

static GtkWidgetClass *parent_class = NULL;

static int signals[LAST_SIGNAL] = { 0 };

static CDDrive *
get_drive (OGMRipDvdChooser *selection, int nr)
{
  GList *item;

  item = g_list_nth (selection->priv->cdroms, nr);

  return item ? item->data : NULL;
}

G_DEFINE_TYPE (OGMRipDvdChooser, ogmrip_dvd_chooser, GTK_TYPE_COMBO_BOX);

static void
ogmrip_dvd_chooser_class_init (OGMRipDvdChooserClass *klass)
{
  GObjectClass *object_class;
  GtkWidgetClass *widget_class;

  object_class = (GObjectClass *) klass;
  widget_class = (GtkWidgetClass *) klass;

  parent_class = g_type_class_peek_parent (klass);

  object_class->set_property = ogmrip_dvd_chooser_set_property;
  object_class->get_property = ogmrip_dvd_chooser_get_property;
  object_class->finalize     = ogmrip_dvd_chooser_finalize;

  g_object_class_install_property (object_class, PROP_DEVICE, 
      g_param_spec_string ("device", NULL, NULL, NULL, G_PARAM_READWRITE));

  signals[DEVICE_CHANGED] = g_signal_new ("device-changed", G_TYPE_FROM_CLASS (object_class), 
      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (OGMRipDvdChooserClass, device_changed), 
      NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
}

static void
combo_device_changed (GtkComboBox *combo, OGMRipDvdChooser *chooser)
{
  CDDrive *drive;
  int i;

  i = gtk_combo_box_get_active (GTK_COMBO_BOX (chooser));

  if (i < 0)
    g_signal_emit (G_OBJECT (chooser), signals[DEVICE_CHANGED], 0, NULL);
  else
  {
    drive = get_drive (chooser, i);
    if (drive)
      g_signal_emit (G_OBJECT (chooser), signals[DEVICE_CHANGED], 0, drive->device);
  }
}

static void
dvd_combo_box (OGMRipDvdChooser *chooser)
{
  GList *item;
  CDDrive *drive;

  chooser->priv->cdroms = scan_for_cdroms (FALSE, FALSE);

  for (item = chooser->priv->cdroms; item; item = item->next)
  {
    drive = item->data;

    if (drive->display_name == NULL)
      g_warning ("cdrom->display_name != NULL failed");

    if (drive->type & CDDRIVE_TYPE_DVD_DRIVE)
      gtk_combo_box_append_text (GTK_COMBO_BOX (chooser), 
          drive->display_name ? drive->display_name : _("Unnamed DVDRom"));
  }
  gtk_combo_box_set_active (GTK_COMBO_BOX (chooser), 0);

  if (chooser->priv->cdroms == NULL)
    gtk_widget_set_sensitive (GTK_WIDGET (chooser), FALSE);
}

static void
ogmrip_dvd_chooser_init (OGMRipDvdChooser *chooser)
{
  GtkCellRenderer *cell;
  GtkListStore *store;

  chooser->priv = g_new0 (OGMRipDvdChooserPrivate, 1);

  store = gtk_list_store_new (1, G_TYPE_STRING);
  gtk_combo_box_set_model (GTK_COMBO_BOX (chooser), GTK_TREE_MODEL (store));

  cell = gtk_cell_renderer_text_new ();
  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (chooser), cell, TRUE);
  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (chooser), cell, "text", 0, NULL);

  dvd_combo_box (chooser);

  g_signal_connect (G_OBJECT (chooser), "changed", G_CALLBACK (combo_device_changed), chooser);
}

static void
ogmrip_dvd_chooser_finalize (GObject *object)
{
  GList *item;
  OGMRipDvdChooser *chooser;

  g_return_if_fail (object != NULL);
  g_return_if_fail (OGMRIP_IS_DVD_CHOOSER (object));

  chooser = OGMRIP_DVD_CHOOSER (object);

  item = chooser->priv->cdroms;
  while (item)
  {
    CDDrive *drive = item->data;

    item = g_list_remove (item, drive);
    cd_drive_free (drive);
  }

  g_free (chooser->priv);
  chooser->priv = NULL;

  if (G_OBJECT_CLASS (parent_class)->finalize != NULL)
    (*G_OBJECT_CLASS (parent_class)->finalize) (object);
}

GtkWidget *
ogmrip_dvd_chooser_new (void)
{
  GtkWidget *widget;

  widget = g_object_new (OGMRIP_TYPE_DVD_CHOOSER, NULL);

  return widget;
}

static void
ogmrip_dvd_chooser_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
{
  OGMRipDvdChooser *chooser;

  g_return_if_fail (OGMRIP_IS_DVD_CHOOSER (object));

  chooser = OGMRIP_DVD_CHOOSER (object);

  switch (property_id)
  {
    case PROP_DEVICE:
      ogmrip_dvd_chooser_set_device (chooser, g_value_get_string (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
  }
}

static void
ogmrip_dvd_chooser_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
{
  OGMRipDvdChooser *chooser;

  g_return_if_fail (OGMRIP_IS_DVD_CHOOSER (object));

  chooser = OGMRIP_DVD_CHOOSER (object);

  switch (property_id)
  {
    case PROP_DEVICE:
      g_value_set_string (value, ogmrip_dvd_chooser_get_device (chooser));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
  }
}

const char *
ogmrip_dvd_chooser_get_default_device (OGMRipDvdChooser *chooser)
{
  GList *item;
  CDDrive *drive;

  g_return_val_if_fail (chooser != NULL, "/dev/dvd");
  g_return_val_if_fail (OGMRIP_IS_DVD_CHOOSER (chooser), "/dev/dvd");

  if (chooser->priv->cdroms == NULL)
    return "/dev/dvd";

  for (item = chooser->priv->cdroms; item; item = item->next)
  {
    drive = item->data;
    if (drive->type & CDDRIVE_TYPE_DVD_DRIVE)
      return drive->device;
  }

  return NULL;
}

void
ogmrip_dvd_chooser_set_device (OGMRipDvdChooser *chooser, const char *device)
{
  GList *item;
  CDDrive *drive;
  int i;

  g_return_if_fail (chooser != NULL);
  g_return_if_fail (OGMRIP_IS_DVD_CHOOSER (chooser));

  for (item = chooser->priv->cdroms, i = 0; item; item = item->next, i++)
  {
    drive = item->data;

    if (strcmp (drive->device, device) == 0)
      break;
  }

  if (item && (drive->type & CDDRIVE_TYPE_DVD_DRIVE))
    gtk_combo_box_set_active (GTK_COMBO_BOX (chooser), i);
  else
  {
    gtk_combo_box_set_active (GTK_COMBO_BOX (chooser), 0);

    if ((drive = get_drive (chooser, 0)))
      g_signal_emit (G_OBJECT (chooser), signals[DEVICE_CHANGED], 0, drive->device);
  }
}

const char *
ogmrip_dvd_chooser_get_device (OGMRipDvdChooser *chooser)
{
  CDDrive *drive;
  int i;

  g_return_val_if_fail (chooser != NULL, NULL);
  g_return_val_if_fail (OGMRIP_IS_DVD_CHOOSER (chooser), NULL);

  i = gtk_combo_box_get_active (GTK_COMBO_BOX (chooser));
  drive = get_drive (chooser, i);

  return drive ? drive->device : NULL;
}

const CDDrive *
ogmrip_dvd_chooser_get_drive (OGMRipDvdChooser *chooser)
{
  CDDrive *drive;
  int i;

  g_return_val_if_fail (chooser != NULL, NULL);
  g_return_val_if_fail (OGMRIP_IS_DVD_CHOOSER (chooser), NULL);

  i = gtk_combo_box_get_active (GTK_COMBO_BOX (chooser));
  drive = get_drive (chooser, i);

  return drive;
}

