/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"

#include "ogmrip-helper.h"

#include <glib/gi18n.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>

#ifdef HAVE_STATVFS
#include <sys/statvfs.h>
#endif

#ifdef __FreeBSD__
#include <sys/cdio.h>
#define CDROMEJECT CDIOCEJECT
#if __FreeBSD_version < 500000
#undef MIN
#undef MAX
#include <sys/param.h>
#include <sys/mount.h>
#endif /* __FreeBSD_version */
#endif /* __FreeBSD__ */

#ifdef __linux__
#include <linux/cdrom.h>
#endif /* __linux__ */

static gchar *tmp_dir = NULL;

GFile *
g_file_open (const gchar *name, GError **error)
{
  GFile *file;

  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  file = g_new0 (GFile, 1);

  file->fd = open (name, O_RDONLY | O_CREAT, 0644);
  if (file->fd < 0)
  {
    g_free (file);
    return NULL;
  }
  file->name = g_strdup (name);

  return file;
}

GFile *
g_file_open_temp (const gchar *tmpl, GError **error)
{
  GFile *file;
  GError *tmp_error = NULL;

  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  file = g_new0 (GFile, 1);

  file->fd = ogmrip_file_open_tmp (tmpl, &file->name, &tmp_error);
  if (file->fd < 0)
  {
    g_propagate_error (error, tmp_error);
    g_free (file);
    return NULL;
  }

  return file;
}

void
g_file_close (GFile **file, gboolean do_unlink)
{
  g_return_if_fail (file != NULL);

  if (*file)
  {
    if (do_unlink && (*file)->name)
      unlink ((*file)->name);

    if ((*file)->name)
      g_free ((*file)->name);

    if ((*file)->fd > 0)
      close ((*file)->fd);

    g_free (*file);

    *file = NULL;
  }
}

glong
g_file_get_size (GFile *file)
{
  struct stat buf;

  g_return_val_if_fail (file != NULL, -1);
  g_return_val_if_fail (g_file_test (file->name, G_FILE_TEST_IS_REGULAR), -1);

  if (stat (file->name, &buf) == -1)
    return -1;

  return buf.st_size;
}

gboolean
g_file_rename (GFile *file, const gchar *new_name, GError **error)
{
  gint fd;

  g_return_val_if_fail (file != NULL, FALSE);
  g_return_val_if_fail (new_name != NULL, FALSE);
  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

  if (strcmp (file->name, new_name) == 0)
    return TRUE;

  if (rename (file->name, new_name) < 0)
    return FALSE;

  fd = open (new_name, O_RDONLY | O_CREAT, 0644);
  if (fd < 0)
    return FALSE;

  close (file->fd);
  file->fd = fd;

  g_free (file->name);
  file->name = g_strdup (new_name);

  return TRUE;
}

gchar *
g_fs_get_mount_point (const gchar *filename)
{
  gchar *cwd, *dirname, *mp = NULL;
  struct stat cur_stat, last_stat;

  g_return_val_if_fail (filename && *filename, NULL);

  cwd = g_get_current_dir ();

  if (g_file_test (filename, G_FILE_TEST_IS_DIR))
    dirname = g_strdup (filename);
  else
    dirname = g_path_get_dirname (filename);

  if (!g_file_test (dirname, G_FILE_TEST_EXISTS))
    goto done;

  if (stat (dirname, &last_stat) < 0)
    goto done;

  if (chdir (dirname) < 0)
    goto done;

  for (;;)
  {
    if (stat ("..", &cur_stat) < 0)
      goto done;

    if (cur_stat.st_dev != last_stat.st_dev || cur_stat.st_ino == last_stat.st_ino)
      break;

    if (chdir ("..") < 0)
      goto done;

    last_stat = cur_stat;
  }

  mp = g_get_current_dir ();

done:
  chdir (cwd);
  g_free (cwd);
  g_free (dirname);

  return mp;
}

gint64
g_fs_space_left (const gchar *filename)
{
  gint status;
  gchar *dirname;
#ifdef HAVE_STATVFS
  struct statvfs statfs_buf;
#else
  struct statfs statfs_buf;
#endif

  g_return_val_if_fail (filename && *filename, -1);

  if (g_file_test (filename, G_FILE_TEST_IS_DIR))
    dirname = g_strdup (filename);
  else
    dirname = g_path_get_dirname (filename);

  if (!g_file_test (dirname, G_FILE_TEST_EXISTS))
  {
    g_free (dirname);
    return -1;
  }

#ifdef HAVE_STATVFS
  status = statvfs (dirname, &statfs_buf);
#else
  status = statfs (dirname, &statfs_buf);
#endif

  g_free (dirname);

  if (status < 0)
    return -1;

#ifdef OGMRIP_DEBUG
  g_message ("Space left on device containing '%s': %lld bytes", 
      filename, (gint64) statfs_buf.f_bsize * (gint64) statfs_buf.f_bavail);
#endif

  return (gint64) statfs_buf.f_bsize * (gint64) statfs_buf.f_bavail;
}

gboolean
g_dir_make (const gchar *path, mode_t mode, GError **error)
{
  const gchar *sep;
  gchar *current;

  g_return_val_if_fail (path && *path, FALSE);
  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

  for (sep = path; *sep; sep ++)
  {
    for (; *sep; sep ++)
      if (*sep == G_DIR_SEPARATOR)
        break;

    if (sep - path > 0)
    {
      current = g_strndup (path, sep - path);
      if (mkdir (current, mode) < 0)
      {
        g_set_error (error, OGMRIP_ERROR, OGMRIP_ERROR_INTERNAL, 
            _("Could not create directory %s: %s"), path, g_strerror (errno));
        g_free (current);
        return FALSE;
      }
      g_free (current);
    }
  }
  return TRUE;
}

gint
gconf_client_get_int_default (GConfClient *client, const gchar *key, gint def)
{
  GConfValue* val;

  val = gconf_client_get (client, key, NULL);
  if (!val)
    gconf_client_set_int (client, key, def, NULL);
  else
  {
    def = gconf_value_get_int (val);
    gconf_value_free (val);
  }

  return def;
}

gboolean
gconf_client_get_bool_default (GConfClient *client, const gchar *key, gboolean def)
{
  GConfValue* val;

  val = gconf_client_get (client, key, NULL);
  if (!val)
    gconf_client_set_bool (client, key, def, NULL);
  else
  {
    def = gconf_value_get_bool (val);
    gconf_value_free (val);
  }

  return def;
}

gchar *
gconf_client_get_string_default (GConfClient *client, const gchar *key, const gchar *def)
{
  GConfValue* val;
  gchar *str;

  val = gconf_client_get (client, key, NULL);
  if (!val)
  {
    gconf_client_set_string (client, key, def, NULL);
    str = g_strdup (def);
  }
  else
  {
    str = g_strdup (gconf_value_get_string (val));
    gconf_value_free (val);
  }

  return str;
}

void
gtk_window_set_parent (GtkWindow *window, GtkWindow *parent)
{
  g_return_if_fail (GTK_IS_WINDOW (window));
  g_return_if_fail (GTK_IS_WINDOW (parent));
  g_return_if_fail (window != parent);

  gtk_window_set_transient_for (window, parent);
  gtk_window_set_position (window, GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_gravity (window, GDK_GRAVITY_CENTER);
  gtk_window_set_destroy_with_parent (window, TRUE);
}

void
gtk_window_set_icon_from_stock (GtkWindow *window, const gchar *stock_id)
{
  GdkPixbuf *pixbuf;

  g_return_if_fail (GTK_IS_WINDOW (window));
  g_return_if_fail (stock_id && *stock_id);
  
  pixbuf = gtk_widget_render_icon (GTK_WIDGET (window), stock_id, GTK_ICON_SIZE_DIALOG, NULL);
  gtk_window_set_icon (window, pixbuf);
  gdk_pixbuf_unref (pixbuf);
}

gint
gtk_radio_button_get_active (GtkRadioButton *radio)
{
  GSList *link;
  gint i;

  g_return_val_if_fail (GTK_IS_RADIO_BUTTON (radio), -1);

  link = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
  for (i = 0;  link; i++, link = link->next)
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (link->data)))
      return i;

  return -1;
}

void
gtk_radio_button_set_active (GtkRadioButton *radio, guint index)
{
  GSList *link;
  gint i;

  g_return_if_fail (GTK_IS_RADIO_BUTTON (radio));

  link = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
  for (i = 0;  link; i++, link = link->next)
    if (i == index)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (link->data), TRUE);
}

gboolean
gtk_tree_model_iter_prev (GtkTreeModel *tree_model, GtkTreeIter *iter)
{
  gboolean retval = FALSE;
  GtkTreePath *path;

  path = gtk_tree_model_get_path (tree_model, iter);
  if (path)
  {
    if (gtk_tree_path_prev (path))
      retval = gtk_tree_model_get_iter (tree_model, iter, path);
    gtk_tree_path_free (path);
  }

  return retval;
}

void
gtk_label_set_int (GtkLabel *label, gint value)
{
  gchar *text;

  g_return_if_fail (GTK_IS_LABEL (label));

  text = g_strdup_printf ("%d", value);
  gtk_label_set_text (label, text);
  g_free (text);
}

gint
gtk_label_get_int (GtkLabel *label)
{
  const gchar *text;

  g_return_val_if_fail (GTK_IS_LABEL (label), G_MININT);

  text = gtk_label_get_text (label);
  
  return atoi (text);
}

gboolean
ogmdvd_tray_is_open (const gchar *device)
{
#if defined(__linux__)
  gint fd, status;
#elif defined(__FreeBSD__)
  gint fd, status;
  struct ioc_toc_header h;
#endif

  g_return_val_if_fail (device && *device, FALSE);

  fd = open (device, O_RDONLY | O_NONBLOCK | O_EXCL);
  if (fd < 0)
    return FALSE;

#if defined(__linux__)

  status = ioctl (fd, CDROM_DRIVE_STATUS, CDSL_CURRENT);

  close (fd);

  if (status < 0)
    return FALSE;

  return status == CDS_TRAY_OPEN;

#elif defined(__FreeBSD__)

  status = ioctl (fd, CDIOREADTOCHEADER, &h);
  close (fd);

  return status < 0;

#else

  return FALSE;

#endif
}

gboolean
ogmdvd_tray_open (const gchar *device)
{
  gint fd, status;

  g_return_val_if_fail (device != NULL, FALSE);

  fd = open (device, O_RDONLY | O_NONBLOCK);
  if (fd < 0)
    return FALSE;

#ifdef __FreeBSD__
  ioctl (fd, CDIOCALLOW);
#endif

  status = ioctl (fd, CDROMEJECT);
  if (status < 0)
  {
    close (fd);
    return FALSE;
  }

  close (fd);

  return TRUE;
}

gboolean
ogmdvd_is_valid_drive (const CDDrive *drive)
{
  if (drive->type & CDDRIVE_TYPE_DVD_DRIVE)
    if (access (drive->device, R_OK) == 0)
      return TRUE;

  return FALSE;
}

G_CONST_RETURN gchar *
ogmdvd_get_default_device (void)
{
  static const gchar *device = NULL;

  if (!device)
  {
    GList *drives, *link;
    CDDrive *drive;

    drives = scan_for_cdroms (FALSE, FALSE);
    if (!drives)
      return NULL;

    for (link = drives; link; link = link->next)
    {
      drive = link->data;
      if (ogmdvd_is_valid_drive (drive));
          break;
    }

    if (link)
      device = g_strdup (drive->device);

    g_list_foreach (drives, (GFunc) cd_drive_free, NULL);
    g_list_free (drives);
  }

  return device;
}

gboolean
ogmdvd_is_valid_device (const gchar *device)
{
  gboolean result;
  GList *drives, *link;
  CDDrive *drive;

  g_return_val_if_fail (device && *device, FALSE);

  drives = scan_for_cdroms (FALSE, FALSE);
  for (link = drives; link; link = link->next)
  {
    drive = link->data;
    if (ogmdvd_is_valid_drive (drive))
      break;
  }
  
  result = link != NULL;

  g_list_foreach (drives, (GFunc) cd_drive_free, NULL);
  g_list_free (drives);

  return result;
}

gboolean
ogmdvd_is_valid_media (const gchar *device)
{
  CDMediaType type;
#ifdef __linux__
  gint fd, status;
#endif

  g_return_val_if_fail (device && *device, FALSE);

  type = cd_drive_get_media_type_from_path (device);

  if (type != CD_MEDIA_TYPE_UNKNOWN && type < CD_MEDIA_TYPE_DVD)
    return FALSE;

  if (type == CD_MEDIA_TYPE_UNKNOWN)
    g_warning ("Unknown media type. Assuming it is valid.");

#ifdef __linux__

  fd = open (device, O_RDONLY | O_NONBLOCK | O_EXCL);
  if (fd < 0)
    return FALSE;

  status = ioctl (fd, CDROM_DISC_STATUS, CDSL_CURRENT);
  close (fd);

  if (status < 0)
    return FALSE;

  return status != CDS_NO_INFO && status != CDS_NO_DISC;

#endif

  return TRUE;
}

G_CONST_RETURN gchar *
ogmdvd_check_device (const gchar *device)
{
  if (!device || !ogmdvd_is_valid_device (device))
    device = ogmdvd_get_default_device ();

  if (!device)
    return NULL;

  return device;
}

GQuark
ogmrip_error_quark (void)
{
  static GQuark quark = 0;

  if (quark == 0)
    quark = g_quark_from_static_string ("ogmrip-error-quark");

  return quark;
}

gint
ogmrip_message_dialog (GtkWindow *parent, GtkMessageType type, const gchar *format, ...)
{
  GtkWidget *dialog = NULL;
  gchar *message;
  gint response;
  va_list args;

  g_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), GTK_RESPONSE_NONE);

  va_start (args, format);
  message = g_strdup_vprintf (format, args);
  va_end (args);

  switch (type)
  {
    case GTK_MESSAGE_ERROR:
      dialog = gtk_message_dialog_new (parent, 
          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, 
          type, GTK_BUTTONS_CLOSE, message);
      gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_DIALOG_ERROR);
      break;
    case GTK_MESSAGE_QUESTION:
      dialog = gtk_message_dialog_new (parent, 
          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, 
          type, GTK_BUTTONS_YES_NO, message);
      gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_DIALOG_QUESTION);
      break;
    case GTK_MESSAGE_INFO:
      dialog = gtk_message_dialog_new (parent, 
          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, 
          type, GTK_BUTTONS_CLOSE, message);
      gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_DIALOG_INFO);
      break;
    case GTK_MESSAGE_WARNING:
      dialog = gtk_message_dialog_new (parent, 
          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, 
          type, GTK_BUTTONS_CLOSE, message);
      gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_DIALOG_WARNING);
      break;
    default:
      break;
  }

  g_free (message);

  if (dialog)
  {
    gtk_window_set_gravity (GTK_WINDOW (dialog), GDK_GRAVITY_CENTER);
    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
    gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
    response = gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
    return response;
  }

  return GTK_RESPONSE_NONE;
}

G_CONST_RETURN gchar *
ogmrip_get_tmp_dir (void)
{
  if (!tmp_dir)
    ogmrip_set_tmp_dir (g_get_tmp_dir ());

  return tmp_dir;
}

void
ogmrip_set_tmp_dir (const gchar *dir)
{
  g_return_if_fail (dir != NULL);
  g_return_if_fail (g_file_test (dir, G_FILE_TEST_IS_DIR));

  g_free (tmp_dir);
  tmp_dir = g_strdup (dir);
}

gint
ogmrip_file_open_tmp (const gchar *tmpl, gchar **name_used, GError **error)
{
  const gchar *tmpdir;
  gchar *fulltmpl;
  gint retval;

  g_return_val_if_fail (error == NULL || *error == NULL, -1);

  if (!tmpl)
    tmpl = ".XXXXXX";

  if (!g_str_has_suffix (tmpl, "XXXXXX"))
  {
    g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED, 
        "Template '%s' doesn't end with XXXXXX", tmpl);
    return -1;
  }

  if ((strchr (tmpl, G_DIR_SEPARATOR)) != NULL 
#ifdef G_OS_WIN32
      || (strchr (tmpl, '/') != NULL)
#endif
      )
  {
    g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED, 
        "Template '%s' invalid, should not contain a '/'", tmpl);
    return -1;
  }

  tmpdir = ogmrip_get_tmp_dir ();
  fulltmpl = g_build_filename (tmpdir, tmpl, NULL);

  retval = g_mkstemp (fulltmpl);
  if (retval == -1)
  {
    g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), 
        "Failed to create file '%s': %s", tmpl, g_strerror (errno));
    g_free (fulltmpl);
    return -1;
  }

  if (name_used)
    *name_used = fulltmpl;
  else
    g_free (fulltmpl);

  return retval;
}

