/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"

#include <gnome.h>
#include <glade/glade.h>

#include "ogmrip-progress.h"
#include "ogmrip-helper.h"

#define OGMRIP_GLADE_FILE "ogmrip/ogmrip-progress.glade"

typedef struct
{
  OGMSpSpawn *spawn;
  GtkWidget *dialog;

  GtkWidget *title_label;
  GtkWidget *stage_label;
  GtkWidget *eta_label;

  GtkWidget *stage_progress;
  GtkWidget *total_progress;

  guint stage;
  guint completed;
  gulong start_time;
} DialogData;

static void
ogmrip_progress_responsed (GtkWidget *dialog, gint response_id, DialogData *data)
{
  if (response_id != GTK_RESPONSE_NONE)
  {
    response_id = ogmrip_message_dialog (GTK_WINDOW (dialog), GTK_MESSAGE_QUESTION,
        _("Are you sure you want to cancel the encoding process?"));

    if (response_id == GTK_RESPONSE_YES)
      ogmsp_spawn_cancel (data->spawn);
  }
}

static void
ogmrip_progress_destroyed (GtkWidget *dialog, DialogData *data)
{
  if (data->spawn)
    g_object_unref (data->spawn);

  g_free (data);
}

static gboolean
ogmrip_progress_delete_event (GtkWidget *dialog, GdkEvent *event, gpointer data)
{
  return TRUE;
}

static void
ogmrip_progress_progress (OGMSpSpawn *spawn, gdouble fraction, DialogData *data)
{
  GTimeVal tv;
  gulong eta;
  gchar *str;

  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (data->stage_progress), fraction);

  g_get_current_time (&tv);
  eta = (1.0 - fraction) * (tv.tv_sec - data->start_time) / fraction;

  if (eta >= 3600)
    str = g_strdup_printf ("%ld hour(s) %ld minute(s)", eta / 3600, (eta % 3600) / 60);
  else
    str = g_strdup_printf ("%ld minute(s)", eta / 60);

  gtk_label_set_text (GTK_LABEL (data->eta_label), str);
  g_free (str);

  fraction = (data->completed + fraction) / (gdouble) data->stage;
  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (data->total_progress), CLAMP (fraction, 0.0, 1.0));
}

GtkWidget *
ogmrip_progress_new (const gchar *title, guint stage)
{
  DialogData *data;
  GladeXML *xml;
  GTimeVal tv;

  xml = glade_xml_new (OGMRIP_DATA_DIR "/" OGMRIP_GLADE_FILE, NULL, NULL);
  if (!xml)
  {
    g_warning ("Could not find " OGMRIP_GLADE_FILE);
    return NULL;
  }

  data = g_new0 (DialogData, 1);
  data->completed = -1;
  data->stage = stage;

  g_get_current_time (&tv);
  data->start_time = tv.tv_sec;

  data->dialog = glade_xml_get_widget (xml, "progress-dialog");
  g_signal_connect (data->dialog, "delete-event", G_CALLBACK (ogmrip_progress_delete_event), NULL);
  g_signal_connect (data->dialog, "response", G_CALLBACK (ogmrip_progress_responsed), data);
  g_signal_connect (data->dialog, "destroy", G_CALLBACK (ogmrip_progress_destroyed), data);

  gtk_window_set_icon_from_stock (GTK_WINDOW (data->dialog), GTK_STOCK_EXECUTE);
  g_object_set_data (G_OBJECT (data->dialog), "__dialog_data__", data);

  data->title_label = glade_xml_get_widget (xml, "title-label");
  gtk_label_set_text (GTK_LABEL (data->title_label), title);

  data->stage_label = glade_xml_get_widget (xml, "stage-label");
  data->eta_label = glade_xml_get_widget (xml, "eta-label");

  data->stage_progress = glade_xml_get_widget (xml, "stage-progress");
  data->total_progress = glade_xml_get_widget (xml, "total-progress");

  g_object_set_data (G_OBJECT (data->dialog), "__ogmrip_progress__", data);

  g_object_unref (xml);

  return data->dialog;
}

void
ogmrip_progress_set_spawn (GtkWidget *dialog, OGMSpSpawn *spawn, const gchar *title)
{
  DialogData *data;
  GTimeVal tv;

  data = g_object_get_data (G_OBJECT (dialog), "__dialog_data__");
  data->completed ++;

  if (spawn != data->spawn)
  {
    g_object_ref (spawn);
    if (data->spawn)
      g_object_unref (data->spawn);
    data->spawn = spawn;

    g_signal_connect (spawn, "progress", 
        G_CALLBACK (ogmrip_progress_progress), data);
  }

  if (title)
    gtk_label_set_markup (GTK_LABEL (data->stage_label), title);

  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (data->stage_progress), 0.0);

  g_get_current_time (&tv);
  data->start_time = tv.tv_sec;
}

